package SPVM::Arrays;

1;

=head1 NAME

SPVM::Arrays - Array Utility

=head1 CONSTANT FUNCTIONS

=head1 FUNCTIONS

=head2 copy_of_byte

=head2 copy_of_short

=head2 copy_of_int

=head2 copy_of_long

=head2 copy_of_float

=head2 copy_of_double

