package SPVM::Byte;

1;

=head1 NAME

SPVM::Byte - Byte Utility

=head1 CONSTANT FUNCTIONS

=head2 MIN_VALUE

B<byte MIN_VALUE()>

A constant holding the minimum value a byte can have, C<-128>.

=head2 MAX_VALUE

B<byte MAX_VALUE()>

A constant holding the maximum value a byte can have, C<127>.

=head2 SIZE

B<int SIZE>

The number of bits used to represent a byte value in two's complement binary form.

=head2 BYTES

B<int BYTES()>

The number of bytes used to represent a byte value in two's complement binary form.

=head1 FUNCTIONS

