#ifndef SPVM_CONSTANT_POOL_FIELD_INFO_H
#define SPVM_CONSTANT_POOL_FIELD_INFO_H

#include "spvm_base.h"

// SPVM_CONSTANT_POOL_FIELD_INFO
struct SPVM_constant_pool_field_info {
  int32_t abs_name_id;
  int32_t name_id;
  int32_t index;
  int32_t type_id;
};

#endif
