package SPVM::Long;

1;

=head1 NAME

SPVM::Long - Long Utility

=head1 CONSTANT FUNCTIONS

=head2 MIN_VALUE

B<long MIN_VALUE()>

A constant holding the minimum value a long can have, C<-128>.

=head2 MAX_VALUE

B<long MAX_VALUE()>

A constant holding the maximum value a long can have, C<127>.

=head2 SIZE

B<int SIZE>

The number of bits used to represent a long value in two's complement binary form.

=head2 BYTES

B<int BYTES()>

The number of longs used to represent a long value in two's complement binary form.

=head1 FUNCTIONS

