package SPVM::Short;

1;

=head1 NAME

SPVM::Short - Short Utility

=head1 CONSTANT FUNCTIONS

=head2 MIN_VALUE

B<short MIN_VALUE()>

A constant holding the minimum value a short can have, C<-32768>.

=head2 MAX_VALUE

B<short MAX_VALUE()>

A constant holding the maximum value a short can have, C<32767>.

=head2 SIZE

B<int SIZE>

The number of bits used to represent a short value in two's complement binary form.

=head2 BYTES

B<int BYTES()>

The number of shorts used to represent a short value in two's complement binary form.

=head1 FUNCTIONS

